/**
 * Copyright (c) 2020 Raspberry Pi (Trading) Ltd.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#include <stdio.h>
#include "pico/stdlib.h"
#include "bsp_co5300.h"

#define OLED_WIDTH 466
#define OLED_HEIGHT 466


uint16_t color_index = 0;
uint16_t color[] = {0xf800, 0x07e0, 0x001f};
bool flush_done_flag = true;

void swap_color(uint8_t *color)
{
    uint8_t swap = *color;
    *color = *(color + 1);
    *(color + 1) = swap;
}

void flush_done(void)
{
    flush_done_flag = true;
}

int main()
{

    stdio_init_all();
    uint8_t brightness = 80;
    static bsp_co5300_info_t co5300_info;
    co5300_info.width = OLED_WIDTH;
    co5300_info.height = OLED_HEIGHT;
    co5300_info.x_offset = 6;
    co5300_info.y_offset = 0;
    co5300_info.enabled_dma = true;
    co5300_info.brightness = brightness;
    co5300_info.dma_flush_done_callback = flush_done;
    bsp_co5300_init(&co5300_info);
    for (int i = 0; i < 3; i++)
    {
        swap_color((uint8_t *)&color[i]);
    }
    bsp_co5300_set_window(0, 0, co5300_info.width - 1, co5300_info.height - 1);
    uint16_t color_arr[co5300_info.width * co5300_info.height];
    while (true)
    {
        printf("Hello, world!\n");

        if (co5300_info.enabled_dma)
        {
            while (!flush_done_flag)
                sleep_us(100);
            flush_done_flag = false;
        }

        for (int i = 0; i < co5300_info.width * co5300_info.height; i++)
        {
            color_arr[i] = color[color_index];
        }
        bsp_co5300_flush(color_arr, co5300_info.width * co5300_info.height);
        if (++color_index > 2)
        {
            color_index = 0;
        }
        bsp_co5300_set_brightness(brightness);
        brightness += 5;
        if (brightness >= 100)
        {
            brightness = 0;
        }

        sleep_ms(1000);
    }
}
