/**
 * Copyright (c) 2020 Raspberry Pi (Trading) Ltd.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#include <stdio.h>
#include "pico/stdlib.h"

#include "bsp_i2c.h"

#include "bsp_co5300.h"
#include "bsp_ft6146.h"

#define OLED_WIDTH 466
#define OLED_HEIGHT 466

uint16_t color_index = 0;
uint16_t color[] = {0xf800, 0x07e0, 0x001f};
bool flush_done_flag = true;

void swap_color(uint8_t *color)
{
    uint8_t swap = *color;
    *color = *(color + 1);
    *(color + 1) = swap;
}

void flush_done(void)
{
    flush_done_flag = true;
}

int main()
{
    stdio_init_all();
    bsp_i2c_init();
    static bsp_co5300_info_t co5300_info;
    co5300_info.width = OLED_WIDTH;
    co5300_info.height = OLED_HEIGHT;
    co5300_info.x_offset = 6;
    co5300_info.y_offset = 0;
    co5300_info.brightness = 80;
    co5300_info.enabled_dma = true;
    co5300_info.dma_flush_done_callback = flush_done;
    bsp_co5300_init(&co5300_info);
    bsp_co5300_set_window(0, 0, co5300_info.width - 1, co5300_info.height - 1);

    static bsp_ft6146_info_t ft6146_info;
    ft6146_info.width = OLED_WIDTH;
    ft6146_info.height = OLED_HEIGHT;
    ft6146_info.rotation = 0;
    bsp_ft6146_init(&ft6146_info);

    for (int i = 0; i < 3; i++)
    {
        swap_color((uint8_t *)&color[i]);
    }
    uint16_t color_arr[co5300_info.width * co5300_info.height];

    for (int i = 0; i < co5300_info.width * co5300_info.height; i++)
    {
        color_arr[i] = 0xffff;
    }
    bsp_ft6146_data_t ft6146_data;
    uint16_t flush_count = 0;
    while (true)
    {
        bsp_ft6146_read();
        if (bsp_ft6146_get_touch_data(&ft6146_data))
        {
            for (int i = 0; i < ft6146_data.points; i++)
            {
                printf("point: %d, x:%d, y:%d\r\n", i, ft6146_data.coords[i].x, ft6146_data.coords[i].y);
                if (ft6146_data.coords[i].x > co5300_info.width - 8)
                    ft6146_data.coords[i].x = co5300_info.width - 8;
                if (ft6146_data.coords[i].y > co5300_info.height - 8)
                    ft6146_data.coords[i].y = co5300_info.height - 8;

                for (int w = 0; w < 8; w++)
                {
                    for (int h = 0; h < 8; h++)
                    {
                        *(color_arr + co5300_info.width * (ft6146_data.coords[i].y + h) + ft6146_data.coords[0].x + w) = color[i];
                    }
                }
            }
        }

        if (++flush_count >= 10)
        {
            // printf("flush_count\r\n");
            flush_count = 0;
            if (co5300_info.enabled_dma)
            {
                while (!flush_done_flag)
                    sleep_us(100);
                flush_done_flag = false;
            }
            bsp_co5300_flush(color_arr, co5300_info.width * co5300_info.height);
        }
        sleep_ms(1);
    }
}
