import sys
sys.path.append('./libraries')
import time
from machine import Pin, I2C
from amoled_1inch43 import CO5300, FT6146

TP_SCL = 7
TP_SDA = 6

LCD_WIDTH = 466
LCD_HEIGHT = 466


def display_clear(display, color):
    display.set_windows(0, 0, display.width - 1, display.height - 1)
    buffer = bytearray(display.width * 2)
    for i in range(len(buffer) / 2):
        buffer[2 * i] = color >> 8
        buffer[2 * i + 1] = color & 0xff
    for i in range(display.height):
        display.flush(buffer)

def display_draw_rect(display, x0, y0, x1, y1, color):
    display.set_windows(x0, y0, x1, y1)
    buffer = bytearray((x1 - x0 + 1) * (y1 - y0 + 1) * 2)
    for i in range(len(buffer) / 2):
        buffer[2 * i] = color >> 8
        buffer[2 * i + 1] = color & 0xff
    display.flush(buffer)

if __name__ == '__main__':
    color_arr = [0xf800, 0x07e0, 0x001f, 0xffe0, 0x07ff, 0xfe1f]
    i2c = I2C(id = 1, scl = Pin(TP_SCL), sda = Pin(TP_SDA), freq = 400_000)
    display = CO5300(width = LCD_WIDTH, height = LCD_HEIGHT, offset_x = 6)
    touch = FT6146(i2c, LCD_WIDTH, LCD_HEIGHT)
    display_clear(display, 0xffff)
    display.set_brightness(255)
    while True:
        if touch.read_touch():
            coords =  touch.get_coords()
            if coords != None:
                for i in range(len(coords)):
                    x0 = coords[i]["x"] - 2
                    y0 = coords[i]["y"] - 2
                    x1 = coords[i]["x"] + 2
                    y1 = coords[i]["y"] + 2
                    if x0 < 0:
                        x0 = 0
                    if y0 < 0:
                        y0 = 0
                    if x1 > display.width - 1:
                        x1 = display.width - 1
                    if y1 > display.height - 1:
                        y1 = display.height - 1
                    display_draw_rect(display, x0, y0, x1, y1, color_arr[i])
                    print("point:" + str(i) + ", x:" + str(coords[i]["x"]) + ", y:" + str(coords[i]["y"]))
        time.sleep(0.01)